import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

import 'config_tsg800/barCode.dart';
import 'config_tsg800/configString.dart';

class GertecTsg800 {
  static const MethodChannel _channel = const MethodChannel('gertec_tsg800');

  /// Realiza a leitura da informação contida no cartão.
  /// '''
  /// Retorna a mensagem contida no cartão.
  static Future<String> get lerCartao async {
    String textoCartao;
    try {
      textoCartao = await _channel.invokeMethod('lerNfc');
    } on PlatformException catch (e) {
      print(e.message);
    }
    return textoCartao;
  }

  /// Realiza a gravação de uma mensagem no cartão.
  /// '''
  /// Recebe como parâmetro obrigatório uma variável do tipo String, texto que vai ser gravado.
  static Future<void> gravarCartao({@required String mensagemGravar}) async {
    try {
      await _channel.invokeMethod(
          'gravarNfc', <String, dynamic>{"mensagemGravar": mensagemGravar});
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Realiza a formatação do cartão (Obs: Cuidado para não apagar dados importantes).
  static Future<void> get formatarCartao async {
    try {
      await _channel.invokeMethod('formatarNfc');
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Realiza um teste de leitura e gravação no Cartão.
  static Future<void> get testeCartao async {
    try {
      await _channel.invokeMethod('testeNfc');
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Realiza a impressão de texto.
  /// '''
  /// Recebe como parâmetro obrigatório a mensagem que vai ser impressa.
  /// '''
  /// Recebe como parâmetro o tamanho da fonte.
  /// '''
  /// Recebe como parâmetro uma variável da Classe [Alinhamento], que determina onde o texto vai ser impresso.
  /// Recebe como parâmetro uma variável da Classe [TipofontFamily], que determina a fonte do texto.
  /// Recebe três parâmetros relacionados a estilo de fonte.
  static Future<void> imprimirTexto(
      {@required String texto,
      int tamanhoFonte = 80,
      Alinhamento alinhamento,
      TipofontFamily fontFamily,
      bool negrito = false,
      bool italico = false,
      bool sublinhado = false}) async {
    try {
      await _channel.invokeMethod(
        'imprimirTexto',
        <String, dynamic>{
          "mensagem": texto,
          "alinhar": alinhamento?.getTipoAlinhamento ??
              Alinhamento.centralizado.getTipoAlinhamento,
          "size": tamanhoFonte,
          "font": fontFamily?.getTipoFontFamily ??
              TipofontFamily.monospace.getTipoFontFamily,
          "negrito": negrito,
          "sublinhado": sublinhado,
          "italico": italico
        },
      );
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Realiza a impressão de um código de barra.
  /// '''
  /// Recebe como parâmetro obrigatório a mensagem que vai ser impressa.
  /// '''
  /// Recebe como parâmetro o height e width, que determinam o tamanho do código de barra.
  /// '''
  /// Recebe como parâmetro uma variável da Classe [TipoCodigoImpressao], que determina o tipo de código que vai ser impresso.
  static Future<void> imprimirCodigoDeBarra(
      {@required String texto,
      int height = 320,
      int width = 320,
      TipoCodigoImpressao codeType}) async {
    if (texto.isNotEmpty) {
      try {
        await _channel.invokeMethod('imprimirCodigoDeBarra', <String, dynamic>{
          "height": height,
          "width": width,
          "barCode": codeType?.getTipoCodigoImpressaoName ??
              TipoCodigoImpressao.qrCode.getTipoCodigoImpressaoName,
          "mensagem": texto
        });
      } on PlatformException catch (e) {
        print(e.message);
      }
    }
  }

  /// Realiza a leitura de Codigo de barra.
  /// '''
  /// Recebe como parâmetro obrigatório uma variável da Classe [TipoCodigoLeitura], que determina qual tipo de código vai ser lido.
  /// '''
  /// Retorna o valor do codigo barras lido.
  static Future<String> leitorCodigoDeBarra(
      {@required TipoCodigoLeitura tipoLeitura}) async {
    String codigo;
    try {
      codigo = await _channel.invokeMethod("leitorV1", <String, dynamic>{
        "tipoLeitura": tipoLeitura?.getTipoCodigoLeituraName ??
            TipoCodigoLeitura.qrCode.getTipoCodigoLeituraName,
      });
    } on PlatformException catch (e) {
      print(e.message);
    }
    return codigo;
  }

  /// Raliza a leitura de Codigo de Barras v2.
  /// '''
  /// Recebe como parâmetro obrigatório um imagem convertida na base 64.
  static Future<void> leitorCodigoDeBarraV2() async {
    try {
      await _channel.invokeMethod('leitorV2');
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Raliza a impressão de uma imagem.
  /// '''
  /// Esta versão funciona para ler qualquer tipo de codigo de barra, não precisa receber um tipo.
  static Future<void> imprimirImagem({@required String codigobase64}) async {
    try {
      await _channel.invokeMethod(
        'imprimirImagem',
        <String, dynamic>{
          "codigoImagem": codigobase64,
        },
      );
    } on PlatformException catch (e) {
      print(e.message);
    }
  }
}
